/*
 * @quakeAnimationFrame.java Version 1.1 08/26/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake.gui;

import java.util.Observer;
import java.util.Observable;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import iqstrat.iqstratHeadersListStruct;    // Well Header List Structure

import gui.guiSearchDirectoryFrame;         // GUI Search Directory Frame

import quake.quakeSeisStruct;               // Quake Seismic Data Structure
import quake.quakePlotFilterListStruct;     // Plot Filter List Structure
import quake.gui.quakeAnimationLimitsFrame; // Animation Limits Frame
import quake.gui.quakeTableFrame;           // Seismic Events Table Frame
import quake.quakeSensorsStruct;            // Sensor Data Structure

import plot.plotXYStruct;                   // XY Data Structure
import plot.plotXYListStruct;               // XY Data List Structure
import plot.plotXYAnimationFrame;           // XY Animate Frame.
import plot.plotXYZAnimationFrame;          // XYZ Animate Frame

import util.utilLandGridListStruct;         // TRS Land Grid List Structure

/** Class quakeAnimationFrame
 *  <p> This file will display the brine Sample Animation Panel and will control
 *      the animation.
 *
 *  @version 1.1 08/26/2015
 *  @author  John R. Victorine
 */

public class quakeAnimationFrame extends JFrame implements ActionListener, ChangeListener, Observer
{
  public static final int _OFF     = 0; // Well Symbols OFF
  public static final int _ON      = 1; // Well Symbols ON

  public static final int STOP     = 0;
  public static final int PAUSE    = 1;
  public static final int RUN      = 2;

  public static final int PREVIOUS = 0;
  public static final int NEXT     = 1;

  // Input Variables

  private quakeSeisStruct          stData   = null; // Quake Seismic Data Structure
  private iqstratHeadersListStruct stWells  = null; // headers Information Data Structure
  private utilLandGridListStruct   stGrid   = null; // TRS Land Grid List Structure
  private quakeSensorsStruct       stSensor = null; // CO2 Seismic Sensor Locations Structure

  // Global Frame Variables

  private Observable       notifier = null; // Observalbles Object

  private int iAction = STOP;
  private int iWells  = _ON;
  private int iGrid   = _ON;

  private static final int FPS_MIN  = 1;
  private static final int FPS_MAX  = 10;
  private static final int FPS_INIT = 2;  //initial frames per second

  private int delay = 500; // milliseconds (1000 = 1 Second)

  private guiSearchDirectoryFrame pSearch = null; // Search for Directory

  private String sPath = "";
  private String sName = "";

  private plotXYListStruct      stList = null;
  private plotXYStruct          stPlot = null;
  private plotXYAnimationFrame  pXY    = null;
  private plotXYZAnimationFrame pXYZ   = null;

  private quakeAnimationLimitsFrame pLimits = null;
  private quakeTableFrame           pTable  = null;

  private int iLimits = _OFF;
  private int iTable  = _OFF;

  // Frame Widgets

  private JButton      btnPDF   = null;                // Create PNG Images

  private JButton      btnLimit = new JButton();       // Set Limits Frame
  private JButton      btnEvent = new JButton();       // Set Events Table Frame

  private JButton      btnStop  = new JButton();       // Stop Animation Action
  private JButton      btnPause = new JButton();       // Pause Animation Action
  private JButton      btnRun   = new JButton();       // Run Animation Action

  private JButton      btnPrev  = null;                // Previous Year Action
  private JButton      btnNext  = null;                // Next Year Action

  private JSlider      slider   = null;

  /** Constructor quakeAnimationFrame()
   *  <p> This is the Constructor for this class
   */

  public quakeAnimationFrame( quakeSeisStruct          stData,
                              iqstratHeadersListStruct stWells,
                              quakeSensorsStruct       stSensor,
                              utilLandGridListStruct   stGrid )
  {
    try
    {
      this.stData   = stData;
      this.stWells  = stWells;
      this.stSensor = stSensor;
      this.stGrid   = stGrid;

      jbInit();

   	  stPlot = quake.quakeAnimateUtility.defaultPlotStruct();

      plotData();

      setButtons();
      addWindowListener( new quakeAnimationFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Panel.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	JPanel   pnlTop        = new JPanel();
	JPanel   pnlCenter     = new JPanel();
	JPanel   pnlSlider     = new JPanel();
	JPanel   pnlCenterBTNS = new JPanel();
	JPanel   pnlBase       = new JPanel();
	JPanel   pnlPlay       = new JPanel();
	JPanel   pnlIncPlay    = new JPanel();

    JLabel   sliderLabel   = null;

    // Create an Observable

    notifier = new quakeAnimationFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    this.setTitle( "Plot Animation Control" );
    this.getContentPane().setLayout(new BorderLayout());

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    btnPDF = new JButton(createImageIcon(cmn.cmnStruct.PDF));
    btnPDF.setPreferredSize(new Dimension(29, 29));
    btnPDF.setToolTipText("Save all Panels Displayed as PNG Images");
    btnPDF.addActionListener(this);

    // Center Panel

    pnlCenter.setLayout(new BorderLayout());

	// -- Slider Panel

	pnlSlider.setLayout(new BorderLayout());

    sliderLabel = new JLabel("Frames Per Second", JLabel.CENTER);
    sliderLabel.setAlignmentX(Component.CENTER_ALIGNMENT);

    slider = new JSlider(JSlider.HORIZONTAL, FPS_MIN, FPS_MAX, FPS_INIT);
    slider.setMajorTickSpacing(1);
    slider.setMinorTickSpacing(1);
    slider.setPaintTicks(true);
    slider.setPaintLabels(true);
    slider.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));
    slider.setFont(new Font("Serif", Font.ITALIC, 12));
    slider.addChangeListener(this);

	// -- Center Button Panel

	pnlCenterBTNS.setLayout(new GridLayout());

    btnLimit.setFont(new java.awt.Font("Dialog", 1, 12));
    btnLimit.setText("Set Plot Limits");
    btnLimit.addActionListener(this);

    btnEvent.setFont(new java.awt.Font("Dialog", 1, 12));
    btnEvent.setText("Set Event to Start");
    btnEvent.addActionListener(this);

    // Base Panel

    pnlBase.setLayout(new BorderLayout());
    pnlBase.setBackground(Color.white);

    // -- Play Panel

    btnStop.setFont(new java.awt.Font("Dialog", 1, 13));
    btnStop.setText("Stop");
    btnStop.setEnabled(false);
    btnStop.addActionListener(this);

    btnPause.setFont(new java.awt.Font("Dialog", 1, 13));
    btnPause.setText("Pause");
    btnPause.setEnabled(false);
    btnPause.addActionListener(this);

    btnRun.setFont(new java.awt.Font("Dialog", 1, 13));
    btnRun.setText("Run");
    btnRun.setEnabled(false);
    btnRun.addActionListener(this);

    // -- Increment Play Panel

    pnlIncPlay.setLayout(new BorderLayout());

    btnPrev = new JButton(createImageIcon(cmn.cmnStruct.PREVIOUS));
    btnPrev.setPreferredSize(new Dimension(27, 27));
    btnPrev.setBackground(Color.white);
    btnPrev.setToolTipText("Previous Event - Increment by event");
    btnPrev.setEnabled(false);
    btnPrev.addActionListener(this);

    btnNext = new JButton(createImageIcon(cmn.cmnStruct.NEXT));
    btnNext.setPreferredSize(new Dimension(27, 27));
    btnNext.setBackground(Color.white);
    btnNext.setToolTipText("Next Event - Increment by event");
    btnNext.setEnabled(false);
    btnNext.addActionListener(this);

	// Attach Widget to Frame

    this.getContentPane().add(pnlTop,    BorderLayout.NORTH);
    pnlTop.add(btnPDF,                   BorderLayout.WEST);

    this.getContentPane().add(pnlCenter, BorderLayout.CENTER);
    pnlCenter.add(pnlSlider,             BorderLayout.NORTH);
    pnlSlider.add(sliderLabel,           BorderLayout.NORTH);
    pnlSlider.add(slider,                BorderLayout.CENTER);

    pnlCenter.add(pnlCenterBTNS,         BorderLayout.CENTER);
    pnlCenterBTNS.add(btnLimit,          null);
    pnlCenterBTNS.add(btnEvent,          null);

    this.getContentPane().add(pnlBase,   BorderLayout.SOUTH);
    pnlBase.add(pnlPlay,                 BorderLayout.CENTER);
    pnlPlay.add(btnStop,                 null);
    pnlPlay.add(btnPause,                null);
    pnlPlay.add(btnRun,                  null);

    pnlBase.add(pnlIncPlay,              BorderLayout.EAST);
    pnlIncPlay.add(btnPrev,              BorderLayout.WEST);
    pnlIncPlay.add(btnNext,              BorderLayout.EAST);

    // Display the Frame

//    this.setSize(new Dimension(500, 120));
    this.setSize(new Dimension(500, 220));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

    this.setLocation((d.width  - this.getSize().width) / 2, 10);
                     //(d.height - this.getSize().height) / 2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ===================================================================== *
   * ------------------------- CLOSE METHODS ----------------------------- *
   * ===================================================================== */

  /** METHOD quakeAnimationFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class quakeAnimationFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      close();
      System.exit(0);
    }
  }

  /** Method close()
   *  <p> deallocate memory and close out frame
   */

  public void close()
  {
	notifier = null; // Observalbles Object

    if (stWells != null)
      stWells.delete();
    stWells = null;

    if (stData != null)
      stData.delete();
    stData = null;

    if (stGrid != null)
      stGrid.delete();
    stGrid = null;

    if (stPlot != null)
      stPlot.delete();
    stPlot = null;

    if (stList != null)
      stList.delete();
    stList = null;

    if (stSensor != null)
      stSensor.delete();
    stSensor = null;

    if (pSearch != null)
      pSearch.close();
    pSearch = null;

    if (pXY != null)
      pXY.close();
    pXY = null;

    if (pXYZ != null)
      pXYZ.close();
    pXYZ = null;

    closeLimitsFrame();
    closeTableFrame();

    btnPDF   = null;

    btnLimit = null;
    btnEvent = null;

    btnStop  = null;
    btnPause = null;
    btnRun   = null;

    btnPrev  = null;
    btnNext  = null;

    slider   = null;

    dispose();
  }

  /** Method closeTableFrame()
   *  <p> This method will close the Table Frame
   */

  private void closeTableFrame()
  {
    if (pTable != null)
      pTable.close();
    pTable  = null;

    iTable  = _OFF;
  }

  /** Method closeLimitsFrame()
   *  <p> This method will close the Table Frame
   */

  private void closeLimitsFrame()
  {
    if (pLimits != null)
      pLimits.close();
    pLimits = null;

    iLimits = _OFF;
  }

  /* ===================================================================== *
   * -------------------------- SET METHODS ------------------------------ *
   * ===================================================================== */

  /** Method setButtons()
   *  <p> This method will enable/disable button, change colors
   */

  private void setButtons()
  {
	btnPDF.setEnabled(false);

    btnStop.setEnabled(false);
    btnPause.setEnabled(false);
    btnRun.setEnabled(false);

    btnEvent.setEnabled(false);

    btnPrev.setEnabled(false);
	btnNext.setEnabled(false);

	if ((stData != null) && (iLimits == _OFF))
	{
	  if (stData.iRows > 0)
	  {
        btnStop.setEnabled(true);
        btnPause.setEnabled(true);
        btnRun.setEnabled(true);
	  }
	}

    switch (iAction)
    {
	  case STOP:
        btnStop.setBackground(new Color(255, 255, 0));
        btnPause.setBackground(new Color(180, 180, 180));
        btnRun.setBackground(new Color(180, 180, 180));

        closeTableFrame();
	    break;
	  case PAUSE:
	    btnPDF.setEnabled(true);

        btnStop.setBackground(new Color(180, 180, 180));
        btnPause.setBackground(new Color(255, 255, 0));
        btnRun.setBackground(new Color(180, 180, 180));

        btnEvent.setEnabled(true);

	    btnPrev.setEnabled(true);
	    btnNext.setEnabled(true);
	    break;
	  case RUN:
        btnStop.setBackground(new Color(180, 180, 180));
        btnPause.setBackground(new Color(180, 180, 180));
        btnRun.setBackground(new Color(255, 255, 0));

        closeTableFrame();
	    break;
	}
  }

  /** Method setDelay()
   *  <p> This method will set the fps of the animation
   *  @param delay = the number of seconds delay
   */

  private void setDelay(int delay)
  {
	this.delay = delay;
    if (pXY  != null) { pXY.setDelay( delay ); }
    if (pXYZ != null) { pXYZ.setDelay( delay ); }
  }

  /* ===================================================================== *
   * ---------------------- Display Plot Frames -------------------------- *
   * ===================================================================== */

  /** Method plotData()
   *  <p> This method will build the Animation Frames and display them.
   */

  private void plotData()
  {
    if (stList != null)
      stList.delete();
    stList = null;

    if (pXY != null)
      pXY.close();
    pXY = null;

    if (pXYZ != null)
      pXYZ.close();
    pXYZ = null;

	stList = quake.quakeAnimateUtility.getData( stData, stPlot );

    pXY  = new plotXYAnimationFrame( iWells, iGrid, stList, stWells, stSensor, stGrid );
    pXYZ = new plotXYZAnimationFrame( iWells, iGrid, stList, stWells, stSensor, stGrid );
  }

  /** Method modifyData()
   *  <p> This method will apply the changes to the animation panels
   */

  private void modifyData()
  {
    if (stList != null)
      stList.delete();
    stList = null;

	stList = quake.quakeAnimateUtility.getData( stData, stPlot );

	if (pXY  != null) { pXY.setChanges(iWells, iGrid, stList); }
	if (pXYZ != null) { pXYZ.setChanges(iWells, iGrid, stList); }
  }

  /** Method displayLimits()
   *  <p> This method will display the limits frame
   */

  private void displayLimits()
  {
    closeLimitsFrame();

    pLimits = new quakeAnimationLimitsFrame( notifier, iWells, iGrid, stPlot );

    iLimits = _ON;
  }

  /** Method displayTable()
   * <p> This method will display the Seismic Events Table
   */

  private void displayTable()
  {
    closeTableFrame();

	pTable  = new quakeTableFrame( notifier, stData, stPlot );

	iTable  = _ON;
  }

  /* ===================================================================== *
   * -------------------------- PDF Methods ------------------------------ *
   * ===================================================================== */

  /** Method createPDF()
   *  <p> This method will create a PDF Image of Profile Plot
   */

  private void createPDF()
  {
    if (pXY != null)
    {
      quake.io.quakeIO_Files.createImage( sPath, sName+"_2D", pXY.getImage() );
      quake.io.quakeIO_Files.createHTML( sPath, sName+"_2D" );
    }

    if (pXYZ != null)
    {
      quake.io.quakeIO_Files.createImage( sPath, sName+"_3D", pXYZ.getImage() );
      quake.io.quakeIO_Files.createHTML( sPath, sName+"_3D" );
    }
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnPDF)
    {
  	  if (sPath.length() == 0)
        sPath = quake.io.quakeIO_Files.getHomeDir();

      sName   = new String( "3DM" );
      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".png");
    }

	if (event.getSource() == btnLimit)
	{
//      iAction = STOP;
//      pXY.stopAnimation();
//      pXYZ.stopAnimation();
      iAction = PAUSE;
	  pXY.pauseAnimation();
	  pXYZ.pauseAnimation();

	  displayLimits();
	}

    if (event.getSource() == btnEvent) { displayTable(); }

	if ((pXY != null) && (pXYZ != null))
	{
      if (event.getSource() == btnStop)
      {
	    iAction = STOP;
	    pXY.stopAnimation();
	    pXYZ.stopAnimation();
	  }

      if (event.getSource() == btnPause)
      {
  	    iAction = PAUSE;
	    pXY.pauseAnimation();
	    pXYZ.pauseAnimation();
	  }

      if (event.getSource() == btnRun)
      {
	    iAction = RUN;
	    pXY.startAnimation();
	    pXYZ.startAnimation();
	  }

      if (event.getSource() == btnPrev)
      {
		if (pXY  != null) { pXY .stepAction(PREVIOUS); }
		if (pXYZ != null) { pXYZ.stepAction(PREVIOUS); }
	  }
      if (event.getSource() == btnNext)
      {
		if (pXY  != null) { pXY .stepAction(NEXT); }
		if (pXYZ != null) { pXYZ.stepAction(NEXT); }
	  }
	}

	setButtons();
  }

  /* ===================================================================== *
   * ------------------------ SLIDER ACTIONS ----------------------------- *
   * ===================================================================== */

  /** Method stateChanged()
   * <p> This method will handle the Slider Actions
   * @param e = Slider Changed Event
   */

  public void stateChanged(ChangeEvent e)
  {
    JSlider source = (JSlider)e.getSource();

    if (!source.getValueIsAdjusting())
    {
      int fps = (int)source.getValue();

      delay = 1000 / fps;
      setDelay( delay );
    }
  }

  /* ===================================================================== *
   * ---------------------- OBSERVABLE ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String((String) arg);

    if (sArg.equals("Apply Changes"))
    {
	  iWells = pLimits.getWellStatus();
	  iGrid  = pLimits.getGridStatus();
	  stPlot = quake.quakeAnimateUtility.copy( pLimits.getPlotStruct() );

//	  plotData();
	  modifyData();

      closeLimitsFrame();
      closeTableFrame();
   	}

    if (sArg.equals("Close Limits Frame")) { closeLimitsFrame(); }

    if (sArg.equals("Event Selected"))
    {
	  if (pTable != null)
	  {
		if (pXY  != null) { pXY.setEvent( pTable.getSelectedRow() ); }
		if (pXYZ != null) { pXYZ.setEvent( pTable.getSelectedRow() ); }
	  }
	}

	if (sArg.equals("Close Events Table Frame")) { closeTableFrame(); }

    if (sArg.equals("Directory Path Selected"))
    {
      if (pSearch != null)
      {
        sPath = new String( pSearch.getPath() );
        sName = new String( pSearch.getFile() );
      }

      createPDF();

      if (pSearch != null)
        pSearch.close();
      pSearch = null;
	}

	setButtons();
  }
}

/** CLASS quakeAnimationFrameObservable()
 *  <p> This Class will handle Observable Actions when action occurs on a
 *      frame or panel
 */

class quakeAnimationFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}

/*
 *  @version 1.1 08/26/2015
 *  @author  John Victorine
 */
