/*
 * @quakeDataUtility.java Version 1.1 08/03/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package quake;

import java.sql.Timestamp;
import java.sql.Date;
import java.text.SimpleDateFormat;
import java.text.ParseException;

import quake.quakeDataListStruct;
import quake.quakeDataStruct;

import quake.quakeSeisStruct;

/** CLASS quakeDataUtility
 *  <p> This file creates utility methods for quake sensor data structures.
 *
 *  @version 1.1 08/03/2015
 *  @author  John R. Victorine
 */

public class quakeDataUtility
{
  public static final int _UTC     = 0;
  public static final int _CENTRAL = 1;

  /** Method copyList()
   * <p> This method will copy one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static quakeDataListStruct copyList(quakeDataListStruct stOld)
  {
    quakeDataListStruct stNew = null;

    if (stOld != null)
    {
      stNew        = new quakeDataListStruct();
      stNew.iCount = stOld.iCount;
      stNew.stItem = new quakeDataStruct[stOld.iCount];

      for (int i=0; i<stOld.iCount; i++)
      {
        stNew.stItem[i] = copy(stOld.stItem[i]);
      }
    }

    return (stNew);
  }

  /** Method transfer()
   * <p> This method will transfer one structure to another
   * @param  stOld = the Old List Structure
   * @return stNew = the New List structure
   */

  public static quakeDataListStruct transfer(quakeDataListStruct stOld)
  {
    quakeDataListStruct stNew = null;

    if (stOld != null)
    {
	  stNew = copyList( stOld );

	  stOld.delete();
	  stOld = null;
	}

    return (stNew);
  }

  /** Method copy()
   * <p> This method will copy data from one structure to another
   * @param  stOld = The original Data Structure
   * @return stNew = The new Data Structure
   */

  public static quakeDataStruct copy(quakeDataStruct stOld)
  {
    quakeDataStruct stNew = new quakeDataStruct();

    if (stOld != null)
    {
      stNew.sKEY       = new String(stOld.sKEY);     // Primary Key

      // Time and Computed location of seismic event.

      stNew.sUTC       = new String(stOld.sUTC);     // Format: MM/dd/yyyy HH:mm:ss
      stNew.sCentral   = new String(stOld.sCentral); // Format: MM/dd/yyyy HH:mm:ss
      stNew.dMagnitude = stOld.dMagnitude;           // Magnitude of Quake
      stNew.sMagType   = new String(stOld.sMagType); // Magnitude Type
      stNew.dLatitude  = stOld.dLatitude;            // Latitude
      stNew.dLongitude = stOld.dLongitude;           // Longitude
      stNew.depth      = stOld.depth;                // Depth
      stNew.state      = new String(stOld.state);    // State
      stNew.source     = new String(stOld.source);   // Source of Data
    }

    return (stNew);
  }

  /** Method add()
   * <p> This method will add the to an existing list
   * @param  stBlock = The Block List Data Structure
   * @param  st      = The Old List Data Structure
   * @return st      = The new List.
   */

  public static quakeDataListStruct add(
	  quakeDataStruct stBlock,
      quakeDataListStruct st)
  {
    int    i=0;
    int    iRecords = 0;
    int    iCount   = 0;
    quakeDataListStruct stTemp = null;

    if (st != null)
      iRecords = st.iCount+1;
    else
      iRecords = 1;

    stTemp        = new quakeDataListStruct();
    stTemp.stItem = new quakeDataStruct[iRecords];

    if (st != null)
    {
      if (st.iCount > 0)
      {
        for (i=0; i<st.iCount; i++)
        {
          if (iCount < iRecords)
          {
            stTemp.stItem[iCount] = copy(st.stItem[i]);
            iCount++;
          }
        }
      }

      st.delete();
    }

    stTemp.stItem[iCount] = new quakeDataStruct();
    stTemp.stItem[iCount] = copy(stBlock);
    iCount++;

    stTemp.iCount = iCount;

    st        = new quakeDataListStruct();
    st.stItem = new quakeDataStruct[iCount];
    st.iCount = iCount;

    for (i=0; i<stTemp.iCount; i++)
    {
      st.stItem[i] = copy(stTemp.stItem[i]);
    }

    stTemp.delete();

    return (st);
  }

  /** Method modify()
   * <p> This method will modify a data structure in an existing list
   * @param  sKEY    = The Primary Key of the data to be removed
   * @param  stBlock = The structure that will be replaced
   * @param  st      = The Old List Data Structure
   * @return st      = The new List.
   */

  public static quakeDataListStruct modify(
	  String sKEY,
      quakeDataStruct stBlock,
      quakeDataListStruct st)
  {
    int i=0;
    quakeDataListStruct stTemp = null;

    if (st != null)
    {
      stTemp        = new quakeDataListStruct();
      stTemp.stItem = new quakeDataStruct[st.iCount];
      stTemp.iCount = st.iCount;

      if (st.iCount > 0)
      {
        for (i=0; i<st.iCount; i++)
        {
          if (sKEY.equals(st.stItem[i].sKEY))
          {
            stTemp.stItem[i] = copy(stBlock);
          }
          else
          {
            stTemp.stItem[i] = copy(st.stItem[i]);
          }
        }
      }

      st.delete();

      st        = new quakeDataListStruct();
      st.stItem = new quakeDataStruct[stTemp.iCount];
      st.iCount = stTemp.iCount;

      for (i=0; i<stTemp.iCount; i++)
      {
        st.stItem[i] = copy(stTemp.stItem[i]);
      }

      stTemp.delete();
    }

    return (st);
  }

  /** Method remove()
   * <p> This method will remove a data structure from an existing list
   * @param  sKEY   = The Primary Key of the data to be removed
   * @param  st     = The Old List Data Structure
   * @return st     = The new List.
   */

  public static quakeDataListStruct remove(
	  String sKEY,
	  quakeDataListStruct st )
  {
    int i=0;
    int    iRecords = 0;
    int    iCount   = 0;
    int    iFound   = -1;
    quakeDataListStruct stTemp = null;

    if (st.iCount == 1)
    {
      st.delete();
      st = null;
    }
    else
    {
      if (st != null)
      {
        iRecords = st.iCount-1;

        stTemp        = new quakeDataListStruct();
        stTemp.stItem = new quakeDataStruct[iRecords];

        for (i=0; i<st.iCount; i++)
        {
          if (sKEY.equals(st.stItem[i].sKEY))
          {
            iFound = i;
          }
        }
      }

      if (st != null)
      {
        if (st.iCount > 0)
        {
          for (i=0; i<st.iCount; i++)
          {
            if (iCount < iRecords)
            {
              if (i != iFound)
              {
                stTemp.stItem[iCount] = copy(st.stItem[i]);
                iCount++;
              }
            }
          }
        }

        st.delete();
      }

      stTemp.iCount = iCount;

      st        = new quakeDataListStruct();
      st.stItem = new quakeDataStruct[iCount];
      st.iCount = iCount;

      for (i=0; i<stTemp.iCount; i++)
      {
        st.stItem[i] = copy(stTemp.stItem[i]);
      }

      stTemp.delete();
    }

    return (st);
  }

  /** Method quakeGetSeismic()
   *  <p> This method will get the Seismic Data Structure from the Quake data Structure
   *  @param  stList = Quake List Data Structure
   *  @return st     = Seismic Data Structure
   */

  public static quakeSeisStruct quakeGetSeismic(int iZone, quakeDataListStruct stList)
  {
	quakeSeisStruct st     = null;
	int             iCount = 0;

	if (stList != null)
	{
	  if (stList.iCount > 0)
	  {
		st            = new quakeSeisStruct();
        st.ts         = new Timestamp[stList.iCount]; // Time Stamp of data record
        st.mag        = new double[stList.iCount];    // Data record
        st.dLatitude  = new double[stList.iCount];    // Latitude
        st.dLongitude = new double[stList.iCount];    // Longitude
        st.depth      = new double[stList.iCount];    // Depth

        st.src        = new String[stList.iCount];    // Source

        for (int i=0; i<stList.iCount; i++)
        {
		  if (iCount < stList.iCount)
		  {
            st.mag[iCount]        = stList.stItem[i].dMagnitude;   // Magnitude
            st.dLatitude[iCount]  = stList.stItem[i].dLatitude;    // Latitude
            st.dLongitude[iCount] = stList.stItem[i].dLongitude;   // Longitude
            st.depth[iCount]      = -1.0 * stList.stItem[i].depth; // Depth

            st.src[iCount]        = new String(stList.stItem[i].source); // Source

            switch (iZone)
            {
              case _UTC:
                st.ts[iCount] = getTimeStamp( stList.stItem[i].sUTC );
                break;

              case _CENTRAL:
                st.ts[iCount] = getTimeStamp( stList.stItem[i].sCentral );
                break;
		    }

            if (i==0)
            {
              st.dMaximum = st.mag[iCount]; // Maximum Data Value
              st.dMinimum = st.mag[iCount]; // Minimum Data Value
			  st.tsStart  = new Timestamp(st.ts[iCount].getTime()); // Starting Date
			  st.tsEnd    = new Timestamp(st.ts[iCount].getTime()); // Ending Date
			}
			else
			{
			  st.tsEnd   = new Timestamp(st.ts[iCount].getTime()); // Starting Date
			}

			if (st.dMinimum > st.mag[iCount])
			  st.dMinimum = st.mag[iCount];

			if (st.dMaximum < st.mag[iCount])
			  st.dMaximum = st.mag[iCount];
//if (st.ts[iCount] != null)
//System.out.println(iCount+" "+st.ts[iCount].toString()+" "+st.data[iCount]);
		    iCount++;
		  }
		}
//System.out.println(iCount+" "+st.dMinimum+" "+st.dMaximum+" "+
//                   st.tsStart.toString()+" "+st.tsEnd.toString());
		st.iRows = iCount;
	  }
	}

	return (st);
  }

  /** Method getTimeStamp( String str )
   *  <p> This method will get the Time Stamp from the Raw Earthquake Data Time.
   *      The Raw Earthquake ORACLE Data Time with Format "MM/DD/YYYY HH24:MI:SS".
   *  @param  sDate = The Raw Earthquake ORACLE Data Time
   *  @return ts    = Timestamp with the Format "yyyy-MM-dd HH:mm:ss.SSSSSS".
   */

   public static Timestamp getTimeStamp( String stng )
   {
	 Timestamp ts = null;
     int    i = 0;
     String tokens[]    = null;
     String str         = "";
     String sDelimiter  = new String("[ ]+");
     String sDelimiter1 = new String("[/]+");
     String sDelimiter2 = new String("[:]+");
     String sDate       = "";
     String sTime       = "";
     String date[]      = { "", "", "" };
     String time[]      = { "", "", "" };
     int    iContinue   = 0;
     double d           = 0.0;
     int    id          = 0;

     SimpleDateFormat dfm   = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSSSSS");
     String           sText = "";
     long             ltime = (long) 0;

	 if (stng != null)
	 {
	   if (stng.length() > 0)
	   {
		 str    = new String(stng);
         tokens = str.split( sDelimiter );
         if (tokens.length > 0)
           sDate = new String(tokens[0].trim());

         tokens = sDate.split( sDelimiter1 );
         if (tokens.length == 3)
         {
  	       for (i=0; i<3; i++)
	       {
	         date[i] = new String(tokens[i]);
	       }

	       iContinue = 1;

           if (iContinue == 1)
	       {
	         iContinue = 0;

   		     str    = new String(stng);
	         tokens = str.split( sDelimiter );
	         if (tokens.length > 0)
	           sTime = new String(tokens[1].trim());

             tokens = sTime.split( sDelimiter2 );
             if (tokens.length == 3)
             {
  	           for (i=0; i<3; i++)
  	           {
	             time[i] = new String(tokens[i]);
	           }

	           iContinue = 1;

	           if (iContinue == 1)
	           {

                 sText = new String( date[2] + "-" +  // Year
  		                             date[0] + "-" +  // Month
		                             date[1] + " " +  // Day
		                             time[0] + ":" +  // 24 Hour
		                             time[1] + ":" +  // Minute
		                             time[2] + ".0"); // Seconds

                 try
                 {
                   ltime = dfm.parse(sText).getTime();
                   ts    = new Timestamp( ltime );
	             }
                 catch (ParseException e)
                 {
                   e.printStackTrace();
                 }

			   }
			 }
		   }
		 }
	   }
	 }

	 return (ts);
  }
}

/*
 *  @version 1.1 08/03/2015
 *  @author  John Victorine
 */
